<?php
namespace Teknisa\PLA\Modules\Cardapio\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Doctrine\DBAL\Connection;
use Zeedhi\Framework\DataSource\DataSet;
use Exception;

class Pla07300CalendarioDeCardapio extends \Teknisa\GER\Controllers\TeknisaCrud {

    protected $crudService;
    protected $parameterBag;
    protected $environment;
    protected $connection;
    
    
    public function __construct($crudService,$parameterBag,$connection,$environment) {
		$this->crudService = $crudService;
		$this->parameterBag = $parameterBag;
		$this->connection = $connection;
		$this->environment = $environment;
	}
    
	private function verifyGroupOnFilter($conditions) {
	    foreach ($conditions as $condition) {
	       if($condition['columnName'] === 'CDGRUPFILI')
	            return true;
	    }
	}
	
	private function getAllBranchCodesByGroupOrNone($row) {
		if($row['T_CDFILIAL'] === 'L') {
			return $row['CDFILIAL_DESTINO'];
		} else {
			$dataSourceToGetFilteredDataSet = 'pla07300_filial_operador';
			$conditions = array(
				array(
					'columnName' => 'CDOPERADOR',
					'operator' => '=',
					'value' => $this->environment->getCdOperador()
				),
				array(
					'columnName' => 'NRORG',
					'operator' => '=',
					'value' => $this->environment->getNrOrg()
				)
			);
			if(isset($row['CDGRUPFILI']) && $row['CDGRUPFILI'] != "") {
				//all branchs by cdgrupfili,cdoperador,nrorg
				$dataSourceToGetFilteredDataSet = 'pla07300_filgrupfili_filial_filioper';
				$conditions = array_merge($conditions,array(
					array(
						'columnName' => 'CDGRUPFILI',
						'operator' => '=',
						'value' => $row['CDGRUPFILI']
					)
				));
			}
			$dataSetFiliais = $this->crudService->getFilteredDataSet($dataSourceToGetFilteredDataSet,$conditions); 
			$rowsFiliais = $dataSetFiliais->getRows();
			$cdfilialArr = array();
			foreach ($rowsFiliais as $filial) {
				array_push($cdfilialArr,array('CDFILIAL'=>$filial['CDFILIAL']));
			}
			return $cdfilialArr;
		}
	}
	
	private function getRowsRemove($rowsCode,$rowsBase) {
		if(sizeof($rowsBase) == 0) {
			return $rowsBase;
		} else {
			$rowsDelete = array();
			foreach ($rowsCode as $code) {
				foreach ($rowsBase as $base) {
					if($base['CDFILIAL'] == $code['CDFILIAL'] && ($code['DTMESCALENCARD'] == substr($base['DTMESCALENCARD'],0,10))) {
						if(is_null($base['DTPREVLIBECARD'])) {
							array_push($rowsDelete,$base);
						}
					}
				}
			}
			return $rowsDelete;
		}
	}
	
	private function getRowsUpdateInsert($rowsCode,$rowsBase) {
		$rowsInsert = array_filter($rowsCode,function($row){return $row['DTPREVLIBECARD'];});
		foreach ($rowsInsert as $key=>$code) {
			foreach ($rowsBase as $base) {
				if($base['CDFILIAL'] == $code['CDFILIAL'] && ($code['DTMESCALENCARD'] == substr($base['DTMESCALENCARD'],0,10))) {
					$rowsInsert[$key]['__is_new'] = false;
					$rowsInsert[$key]['changed'] = true;
				}
			}
			if(!isset($rowsInsert[$key]['__is_new'])) {
				$rowsInsert[$key]['__is_new'] = true;
			}
		}
		return $rowsInsert;
	}
	
	public function exportaCalenCard(DTO\Request $request, DTO\Response $response) {
		$row = $request->getRow();
		
		$row['CDFILIAL'] = $this->getAllBranchCodesByGroupOrNone($row);
		
		$allRows = array();
		$filiaisParaExportar = array();
		$dtmescalencard = array();
		
		foreach ($row['CDFILIAL'] as $filial) {
			array_push($filiaisParaExportar,$filial['CDFILIAL']);
			foreach ($row['CALENDARIO'] as $calendario) {
				array_push($dtmescalencard,$calendario['DTMESCALENCARD']);
				array_push($allRows, array(
					'CDFILIAL' => $filial['CDFILIAL'],
					'DTMESCALENCARD'  => $calendario['DTMESCALENCARD'],
					'DTPREVLIBECARD'  => isset($calendario['DTPREVLIBECARD'])  ? $calendario['DTPREVLIBECARD']  : null,
					'DTLIMAPROVFILI'  => isset($calendario['DTLIMAPROVFILI'])  ? $calendario['DTLIMAPROVFILI']  : null,
					'DTLIMAPROVMATR'  => isset($calendario['DTLIMAPROVMATR'])  ? $calendario['DTLIMAPROVMATR']  : null,
					'QTDIAPRORAPROV'  => isset($calendario['QTDIAPRORAPROV'])  ? $calendario['QTDIAPRORAPROV']  : null,
					'DTBASCONCUSPLAN' => isset($calendario['DTBASCONCUSPLAN']) ? $calendario['DTBASCONCUSPLAN'] : null,
				));
			}
		}
		
		$conditions = [
			[
				'columnName' => 'CDFILIAL',
				'operator' => 'IN',
				'value' => $filiaisParaExportar
			],
			[
				'columnName' => 'DTMESCALENCARD',
				'operator' => 'IN',
				'value' => array_unique($dtmescalencard)
			],
			[
				'columnName' => 'NRORG',
				'operator' => '=',
				'value' => $this->environment->getNrOrg()
			]
		];
		
		$row['CDFILIAL'] = $filiaisParaExportar;
		$rowsFilteredDataSet = $this->crudService->getFilteredDataSet('calencard',$conditions)->getRows();
		//Simplificar em um método para retornas os tres possiveis tipos de row
		$rowsRemove = $this->getRowsRemove($allRows,$rowsFilteredDataSet); #Recebe as rows com as chaves(dtmescalencard,cdfilial) com valor e o resto sem
		if(sizeof($rowsRemove)!= 0)
			$this->crudService->delete($response,new DataSet('calencard',$rowsRemove));
		$rowsUpdate = $this->getRowsUpdateInsert($allRows,$rowsFilteredDataSet); #Recebe as rows com valores nos campos alem das chaves e __is_new false;
		$this->dataSourceNameToSave = 'calencard';
		$dataSave = new DataSet('calencard', $rowsUpdate);
        $requestSave = new DTO\Request\DataSet($dataSave, $request->getMethod(), $request->getRoutePath(), $request->getUserId());
        parent::save($requestSave, $response);
	}
	
	public function prorrogaCalenCard(DTO\Request $request, DTO\Response $response) {
		try {
			$row = $request->getRow();
			$row['CDFILIAL'] = $this->getAllBranchCodesByGroupOrNone($row);
			$rows = array();
			$filiaisParaProrrogar = array();
			foreach ($row['CDFILIAL'] as $filial) {
				array_push($filiaisParaProrrogar,$filial['CDFILIAL']);
			}
			$dtmescalencard = '01/' . $row['MESAPRORAPROV']. ' 00:00:00';
			$conditions = array(
				array(
					'columnName' => 'CDFILIAL',
					'operator' => 'IN',
					'value' => $filiaisParaProrrogar
				),
				array(
					'columnName' => 'DTMESCALENCARD',
					'operator' => '=',
					'value' => $dtmescalencard
				),
				array(
					'columnName' => 'NRORG',
					'operator' => '=',
					'value' => $this->environment->getNrOrg()
				)
			);
			$rowsToUpdate = $this->crudService->getFilteredDataSet('calencard',$conditions)->getRows();
			foreach ($rowsToUpdate as $key=>$rowToUpdate) {
				$rowsToUpdate[$key]['QTDIAPRORAPROV'] = floatval($row['QTDIAPRORAPROV']);
				$rowsToUpdate[$key]['changed'] = true;
			}
			$this->dataSourceNameToSave = 'calencard';			
			$dataSave = new DataSet('calencard',$rowsToUpdate);
	        $requestSave = new DTO\Request\DataSet($dataSave, $request->getMethod(), $request->getRoutePath(), $request->getUserId());
	        parent::save($requestSave, $response);
			
		}
		catch (\Exception $e){
            $response->setError(new Error("erroProcessoExtensao". $e->getMessage(), 500));
        }
	}

	public function findFilialByGroupOrNone(DTO\Request\Filter $request, DTO\Response $response) {
        $criteria = $request->getFilterCriteria();
        $conditions = $criteria->getConditions();
        if($this->verifyGroupOnFilter($conditions)) {
            $criteria->setDataSourceName('pla07300_filgrupfili_filial_filioper');
        } else {
            $criteria->setDataSourceName('pla07300_filial_operador');
        }
        $criteria = $this->crudService->getPopulatedFilterCriteria($criteria,
        									array("CDOPERADOR","NRORG"));
		$this->crudService->find($response, $criteria);
	}
	
	
	public function salvaLogCalencard(DTO\Request $request, DTO\Response $response){
		$row = $request->getRow();
		$query = "INSERT INTO 
				LOGCALENCARD(
					DTALTCALENCARD, 
					DTMESCALENCARD, 
					CDFILIAL, 
					CDOPERADOR, 
					DTPREVLIBECARDANT, 
					DTLIMAPROVFILIANT, 
					QTDIAPRORAPROVANT, 
					DTLIMAPROVMATRANT, 
					DTPREVLIBECARDDEP, 
					DTLIMAPROVFILIDEP, 
					QTDIAPRORAPROVDEP, 
					DTLIMAPROVMATRDEP 
				) VALUES ( 
					:DTALTCALENCARD, 
					:DTMESCALENCARD, 
					:CDFILIAL, 
					:CDOPERADOR, 
					:DTPREVLIBECARDANT, 
					:DTLIMAPROVFILIANT, 
					:QTDIAPRORAPROVANT, 
					:DTLIMAPROVMATRANT, 
					:DTPREVLIBECARDDEP, 
					:DTLIMAPROVFILIDEP, 
					:QTDIAPRORAPROVDEP, 
					:DTLIMAPROVMATRDEP)";

		if(isset($row['__is_new']) && $row['__is_new']){
			
			$params = array(
				"DTALTCALENCARD" => date('d/m/Y h:i:s'),
				"DTMESCALENCARD" => $row['DTMESCALENCARD'],
                "CDFILIAL" => $row['CDFILIAL'],
                "CDOPERADOR" => $this->environment->getCdOperador(),
                "DTPREVLIBECARDANT" => '',
                "DTLIMAPROVFILIANT" => '',
                "QTDIAPRORAPROVANT" => '',
                "DTLIMAPROVMATRANT" => '',
                "DTPREVLIBECARDDEP" => $row->get('DTPREVLIBECARD'),
                "DTLIMAPROVFILIDEP" => $row->get('DTLIMAPROVFILI'),
                "QTDIAPRORAPROVDEP" => $row->get('QTDIAPRORAPROV'),
                "DTLIMAPROVMATRDEP" => $row->get('DTLIMAPROVMATR')
            );
		} else if(isset($row['__is_new']) && !$row['__is_new']){
			$params = array(
				"DTALTCALENCARD" => date('d/m/Y h:i:s'),
				"DTMESCALENCARD" => (isset($row['oldValuesRegister']['DTMESCALENCARD'])) ? $row['oldValuesRegister']['DTMESCALENCARD'] : '',
                "CDFILIAL" => $row['CDFILIAL'],
                "CDOPERADOR" => $this->environment->getCdOperador(),
	            "DTPREVLIBECARDANT" => (isset($row['oldValuesRegister']['DTPREVLIBECARD'])) ? $row['oldValuesRegister']['DTPREVLIBECARD'] : '',
                "DTLIMAPROVFILIANT" => (isset($row['oldValuesRegister']['DTLIMAPROVFILI'])) ? $row['oldValuesRegister']['DTLIMAPROVFILI'] : '',
                "QTDIAPRORAPROVANT" => (isset($row['oldValuesRegister']['QTDIAPRORAPROV'])) ? $row['oldValuesRegister']['QTDIAPRORAPROV'] : '',
                "DTLIMAPROVMATRANT" => (isset($row['oldValuesRegister']['DTLIMAPROVMATR'])) ? $row['oldValuesRegister']['DTLIMAPROVMATR'] : '',
                "DTPREVLIBECARDDEP" => $row->get('DTPREVLIBECARD'),
                "DTLIMAPROVFILIDEP" => $row->get('DTLIMAPROVFILI'),
                "QTDIAPRORAPROVDEP" => $row->get('QTDIAPRORAPROV'),
                "DTLIMAPROVMATRDEP" => $row->get('DTLIMAPROVMATR')
            );
		} else {
			
			$params = array(
				"DTALTCALENCARD" => date('d/m/Y h:i:s'),
				"DTMESCALENCARD" => $row['oldValuesRegister']['DTMESCALENCARD'],
                "CDFILIAL" => $row['CDFILIAL'],
                "CDOPERADOR" => $this->environment->getCdOperador(),
                "DTPREVLIBECARDANT" => $row['oldValuesRegister']['DTPREVLIBECARD'],
                "DTLIMAPROVFILIANT" => $row['oldValuesRegister']['DTLIMAPROVFILI'],
                "QTDIAPRORAPROVANT" => $row['oldValuesRegister']['QTDIAPRORAPROV'],
                "DTLIMAPROVMATRANT" => $row['oldValuesRegister']['DTLIMAPROVMATR'],
                "DTPREVLIBECARDDEP" => '',
                "DTLIMAPROVFILIDEP" => '',
                "QTDIAPRORAPROVDEP" => '',
                "DTLIMAPROVMATRDEP" => ''
            );
			
		}
		
		try {
            $std = $this-> connection->prepare($query);
            $std->execute($params);
            
        } catch (Exception $e) {
            throw $e;
        }
	}
	public function verificaFeriado (DTO\Request $request, DTO\Response $response){
		$row = $request->getRow();
		$dates = array(
			'DTPREVLIBECARD'  => $row['DTPREVLIBECARD'],
			'DTLIMAPROVFILI'  => $row['DTLIMAPROVFILI'],
			'DTLIMAPROVMATR'  => $row['DTLIMAPROVMATR']
		);
		foreach($dates as $key => $dt){
			$row['DTVERIFYHOLIDAY'] = $dt;
			while($this->isHoliday($row, $response)){
				$date = \DateTime::createFromFormat('d/m/Y', $row['DTVERIFYHOLIDAY']);
				$date->sub(new \DateInterval('P1D'));
				$row['DTVERIFYHOLIDAY'] = $date->format('d/m/Y');
			}
			$dates[$key] = $row['DTVERIFYHOLIDAY'];
		}
		$response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet ('dates', $dates));
	}
	private function isHoliday($row, $response) {
		$params = array(
		  'P_CDFILIAL'   => $row['CDFILIAL'],
          'P_DTFERIFILI' => $row['DTVERIFYHOLIDAY']
		);
		try {
			$sql = "SELECT CDFILIAL, DTFERIFILI
						FROM FERIFILI
						WHERE CDFILIAL = :P_CDFILIAL AND
						DTFERIFILI = :P_DTFERIFILI";
	
			
	
			$query = $this->connection->prepare($sql);
	        $query->execute($params);
	        $dataSet = $query->fetchAll();
	        return !!$dataSet;
		} catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
	}
	
}