<?php
namespace Teknisa\PLA\Modules\Cardapio\Services;
/**
 * 01/06/2015
 * @author Francis
 */

class ImportarCardapio {

    const SOBREPOR = 'S';
    const INCLUIDO_MATRIZ = 'N';
    const ALTERADO_FILIAL = 'S';
    const ALTERADO_MATRIZ = 'R';

	protected $entityManager;
    protected $novoCodigo;
    /** @var Environment */
    protected $environment;
    protected $nrcardapio;

    public function __construct($entityManager, $environment, $novoCodigo) {
        $this->entityManager = $entityManager;
        $this->environment = $environment;
        $this->novoCodigo = $novoCodigo;
    }

    private function retornaObjetoData($stringData) {

        $array = explode("/", $stringData);
        return new \DateTime($array[2]."-". $array[1]."-". $array[0]);

    }

    private function inserirComposicao($idpratoespe, $nrordcnfmes, $cdtipoprat, $nrseqcnfmes, $cdfilauxprat, $cdprato, $vrpeconscard, $filialDestino) {

        $dataAtual = new \DateTime();

        $compcard = new \Teknisa\Model\Planejamento\Entities\Compcard;

        $compcard->setCdfilial($filialDestino);
        $compcard->setNrCardapio($this->nrcardapio);
        $compcard->setNrseqpratcar($nrseqcnfmes);
        $compcard->setNrordpratcar($nrordcnfmes);
        $compcard->setIdpratoespe($idpratoespe);
        $compcard->setCdtipoprat($cdtipoprat);
        $compcard->setCdfilauxprat($cdfilauxprat);
        $compcard->setCdprato($cdprato);
        $compcard->setVrpeconscard($vrpeconscard);
        $compcard->setNrorg($this->environment->getNrOrg());
        $compcard->setDtinclusao($dataAtual);
        $compcard->setDtultatu($dataAtual);
        $compcard->setCdoperinclusao($this->environment->getCdOperador());

        $this->entityManager->persist($compcard);
        $this->entityManager->flush();

    }

    private function retornaComposicoes($nrcardapio, $cdfilial) {

        return $this->entityManager->getRepository('\Teknisa\Model\Planejamento\Entities\Compcard')
            ->findBy(
                array(
                    "nrorg" => $this->environment->getNrOrg(),
                    "nrcardapio" => $nrcardapio,
                    "cdfilial" => $cdfilial
                ),
                array('nrordpratcar' => 'ASC')
            );
    }

    private function retornaConfiguracaoDoCardapio($cdservfili, $dtcardapio, $filialDestino) {

        return $this->entityManager->getRepository('\Teknisa\Model\Planejamento\Entities\Cnfcames')
            ->findBy(
                array(
                    "nrorg" => $this->environment->getNrOrg(),
                    "cdfilial" => $filialDestino,
                    "dtrefecnfmes" => $this->retornaObjetoData($dtcardapio)->modify('first day of'),
                    "cdservfili" => $cdservfili
                ),
                array('nrordcnfmes' => 'ASC')
            );
    }

    private function retornaComposicoesFormatadas($nrcardapio, $cdfilial, $cdservfili, $dtcardapio, $filialDestino) {

        $configuracoes = $this->retornaConfiguracaoDoCardapio($cdservfili, $dtcardapio, $filialDestino);

        $composicoes = $this->retornaComposicoes($nrcardapio, $cdfilial);

        $composicaoFormatada = array();

        foreach ($configuracoes as $configuracao) {

            foreach ($composicoes as $key => $value) {

                if($value->getCdtipoprat() == $configuracao->getCdtipoprat()) {

                    $composicaoFormatada[] = array(
                        'nrseqcnfmes' => $configuracao->getNrseqcnfmes(),
                        'cdtipoprat' => $value->getCdtipoprat(),
                        'cdprato' => $value->getCdprato(),
                        'cdfilauxprat' => $value->getCdfilauxprat(),
                        'vrpeconscard' => $value->getVrpeconscard(),
                        'nrordcnfmes' => $configuracao->getNrordcnfmes()
                    );

                    unset($composicoes[$key]);

                    break;

                }

            }

        }

        return $composicaoFormatada;

    }

    private function inserirCardapio($comensais, $clientes, $funcionarios, $doacao, $dtcardapio, $cdservfili, $filialDestino) {

        $dataAtual = new \DateTime();

        $nrcardapio = $this->novoCodigo->getCodeByTableName('CARDAPIO'.$filialDestino, $this->environment->getNrOrg());

        $cardadia = new \Teknisa\Model\Planejamento\Entities\Cardadia;

        $cardadia->setCdfilial($filialDestino);
        $cardadia->setNrCardapio($nrcardapio);
        $cardadia->setCdservfili($cdservfili);
        $cardadia->setDtcardapio($this->retornaObjetoData($dtcardapio));
        $cardadia->setDtmescard($this->retornaObjetoData($dtcardapio)->modify('first day of'));
        $cardadia->setCdoperador($this->environment->getCdOperador());
        $cardadia->setQtcliesticar($clientes);
        $cardadia->setQtfunesticar($funcionarios);
        $cardadia->setQtdoaesticar($doacao);
        $cardadia->setQtcomesticar($comensais);
        $cardadia->setNrorg($this->environment->getNrOrg());
        $cardadia->setDtinclusao($dataAtual);
        $cardadia->setDtultatu($dataAtual);
        $cardadia->setCdoperinclusao($this->environment->getCdOperador());

        $this->entityManager->persist($cardadia);
        $this->entityManager->flush();

        $this->nrcardapio = $nrcardapio;

    }

    private function retornaEstimados($cdservfili, $diaSemana, $cdfilial) {

        $servico = $this->entityManager->getRepository('\Teknisa\Model\Planejamento\Entities\Servico')
            ->findOneBy(
                array(
                    "nrorg" => $this->environment->getNrOrg(),
                    "cdfilial" => $cdfilial,
                    "cdservfili" => $cdservfili
                )
            );

        if($servico) {

            $estimados = array();

            switch ($diaSemana) {

                case "DOM":

                    $estimados[] = array(
                        "QTCLIESTICAR" => $servico->getQtcliestidom(),
                        "QTFUNESTICAR" => $servico->getQtfunestidom(),
                        "QTDOAESTICAR" => $servico->getQtdoaestidom(),
                        "QTCOMESTICAR" => $servico->getQtcomestidom()
                    );

                    break;

                case "SEG":

                    $estimados[] = array(
                        "QTCLIESTICAR" => $servico->getQtcliestiseg(),
                        "QTFUNESTICAR" => $servico->getQtfunestiseg(),
                        "QTDOAESTICAR" => $servico->getQtdoaestiseg(),
                        "QTCOMESTICAR" => $servico->getQtcomestiseg()
                    );

                    break;

                case "TER":

                    $estimados[] = array(
                        "QTCLIESTICAR" => $servico->getQtcliestiter(),
                        "QTFUNESTICAR" => $servico->getQtfunestiter(),
                        "QTDOAESTICAR" => $servico->getQtdoaestiter(),
                        "QTCOMESTICAR" => $servico->getQtcomestiter()
                    );

                    break;

                case "QUA":

                    $estimados[] = array(
                        "QTCLIESTICAR" => $servico->getQtcliestiqua(),
                        "QTFUNESTICAR" => $servico->getQtfunestiqua(),
                        "QTDOAESTICAR" => $servico->getQtdoaestiqua(),
                        "QTCOMESTICAR" => $servico->getQtcomestiqua()
                    );

                    break;

                case "QUI":

                    $estimados[] = array(
                        "QTCLIESTICAR" => $servico->getQtcliestiqui(),
                        "QTFUNESTICAR" => $servico->getQtfunestiqui(),
                        "QTDOAESTICAR" => $servico->getQtdoaestiqui(),
                        "QTCOMESTICAR" => $servico->getQtcomestiqui()
                    );

                    break;

                case "SEX":

                    $estimados[] = array(
                        "QTCLIESTICAR" => $servico->getQtcliestisex(),
                        "QTFUNESTICAR" => $servico->getQtfunestisex(),
                        "QTDOAESTICAR" => $servico->getQtdoaestisex(),
                        "QTCOMESTICAR" => $servico->getQtcomestisex()
                    );

                    break;

                case "SAB":

                    $estimados[] = array(
                        "QTCLIESTICAR" => $servico->getQtcliestisab(),
                        "QTFUNESTICAR" => $servico->getQtfunestisab(),
                        "QTDOAESTICAR" => $servico->getQtdoaestisab(),
                        "QTCOMESTICAR" => $servico->getQtcomestisab()
                    );

                    break;

            }

            return $estimados[0];

        }

    }

    private function retornaCardapioDoDia($cdservfili, $dtcardapio, $filialDestino) {

        return $this->entityManager->getRepository('\Teknisa\Model\Planejamento\Entities\Cardadia')
            ->findOneBy(
                array(
                    "nrorg" => $this->environment->getNrOrg(),
                    "cdfilial" => $filialDestino,
                    "cdservfili" => $cdservfili,
                    "dtcardapio" => $this->retornaObjetoData($dtcardapio)
                )
            );

    }

    private function retornaComposicaoDoCardapio($nrseqpratcar, $filialDestino) {

        return $this->entityManager->getRepository('\Teknisa\Model\Planejamento\Entities\Compcard')
            ->findOneBy(
                array(
                    "nrorg" => $this->environment->getNrOrg(),
                    "nrcardapio" => $this->nrcardapio,
                    "nrseqpratcar" => $nrseqpratcar,
                    "cdfilial" => $filialDestino
                )
            );

    }

    private function removerComposicao($cdfilial, $nrordpratcar) {

        $composicao = $this->entityManager->getRepository('\Teknisa\Model\Planejamento\Entities\Compcard')
            ->findOneBy(
                array(
                    "nrorg" => $this->environment->getNrOrg(),
                    "nrcardapio" => $this->nrcardapio,
                    "nrordpratcar" => $nrordpratcar,
                    "cdfilial" => $cdfilial
                )
            );

        $this->entityManager->remove($composicao);
        $this->entityManager->flush();

    }

    private function retornarComposicaoDestino($cdfilial, $nrseqpratcar) {

        return $this->entityManager->getRepository('\Teknisa\Model\Planejamento\Entities\Compcard')
            ->findOneBy(
                array(
                    "nrorg" => $this->environment->getNrOrg(),
                    "nrcardapio" => $this->nrcardapio,
                    "nrseqpratcar" => $nrseqpratcar,
                    "cdfilial" => $cdfilial
                )
            );

    }

    private function importarComposicao($montagem, $idcarddescentr, $idpratoespe, $nrseqcnfmes, $vrpeconscard, $cdprato, $cdfilauxprat, $nrordcnfmes, $cdtipoprat, $filialDestino) {

        $composicaoDestino = $this->retornaComposicaoDoCardapio($nrseqcnfmes, $filialDestino);

        if($composicaoDestino && $montagem == self::SOBREPOR) {

            if($idpratoespe == self::ALTERADO_FILIAL ||
                $idpratoespe == self::ALTERADO_MATRIZ &&
                $idcarddescentr) {

                $composicaoDestino->setVrpeconscard($vrpeconscard);

                if($composicaoDestino->getIdpratoespe() == self::ALTERADO_MATRIZ ||
                    $composicaoDestino->getIdpratoespe() == self::ALTERADO_FILIAL) {

                    if($composicaoDestino->getCdpratomat() == $cdprato &&
                        $composicaoDestino->getCdfilauxpratmat() == $cdfilauxprat) {

                        $composicaoDestino->setCdpratomat(null);
                        $composicaoDestino->setCdfilauxpratmat(null);
                        $composicaoDestino->setCdprato($cdprato);
                        $composicaoDestino->setCdfilauxprat($cdfilauxprat);
                        $composicaoDestino->setIdpratoespe(self::INCLUIDO_MATRIZ);

                    }else {

                        $composicaoDestino->setCdprato($cdprato);
                        $composicaoDestino->setCdfilauxprat($cdfilauxprat);
                        $composicaoDestino->setIdpratoespe($idpratoespe);

                    }

                }else if($composicaoDestino->getIdpratoespe() == self::INCLUIDO_MATRIZ) {

                    if($composicaoDestino->getCdprato() != $cdprato ||
                        $composicaoDestino->getCdfilauxprat() != $cdfilauxprat) {

                        $composicaoDestino->setCdpratomat($composicaoDestino->getCdprato());
                        $composicaoDestino->setCdfilauxpratmat($composicaoDestino->getCdfilauxprat());
                        $composicaoDestino->setCdprato($cdprato);
                        $composicaoDestino->setCdfilauxprat($cdfilauxprat);
                        $composicaoDestino->setIdpratoespe(self::ALTERADO_FILIAL);

                    }

                }

                $this->entityManager->persist($composicaoDestino);
                $this->entityManager->flush();

            }else {

                $this->entityManager->remove($composicaoDestino);
                $this->entityManager->flush();

                $this->inserirComposicao(self::INCLUIDO_MATRIZ, $nrordcnfmes, $cdtipoprat, $nrseqcnfmes, $cdfilauxprat, $cdprato, $vrpeconscard, $filialDestino);

            }

        }else if(!$composicaoDestino) {

            $this->inserirComposicao($idpratoespe, $nrordcnfmes, $cdtipoprat, $nrseqcnfmes, $cdfilauxprat, $cdprato, $vrpeconscard, $filialDestino);

        }

    }

    public function importarCardapioDoDia($idpratoespe, $idcarddescentr, $dataCardapioOrigem, $montagem, $servicoDestino, $servicoOrigem,
                                                                        $dataCardapioDestino, $diaSemanaDestino, $nrCardapioOrigem, $unidadeOrigem, $filialDestino) {

        $cardadia = $this->retornaCardapioDoDia($servicoDestino, $dataCardapioDestino, $filialDestino);
       
        if(!$cardadia) {

            $composicaoOrigem = $this->entityManager->getRepository('\Teknisa\Model\Planejamento\Entities\Compcard')
                ->findOneBy(
                    array(
                        "nrorg" => $this->environment->getNrOrg(),
                        "nrcardapio" => $nrCardapioOrigem,
                        "cdfilial" => $unidadeOrigem
                    )
                );

            if($composicaoOrigem) {
            
                $estimados = $this->retornaEstimados($servicoDestino, $diaSemanaDestino,$filialDestino);
                
                if($estimados['QTCOMESTICAR'] > 0)
                    $this->inserirCardapio($estimados['QTCOMESTICAR'], $estimados['QTCLIESTICAR'], $estimados['QTFUNESTICAR'],
                                                                                    $estimados['QTDOAESTICAR'], $dataCardapioDestino, $servicoDestino, $filialDestino);
                else
                    $this->nrcardapio = null;

            }

        }
        else
            $this->nrcardapio = $cardadia->getNrcardapio();

        if(isset($this->nrcardapio)) {
            // importar cardapio do mesmo periodo
            if((substr($dataCardapioDestino,3,7) == substr($dataCardapioOrigem,3,7)) &&
                ($unidadeOrigem == $filialDestino) &&
                ($servicoOrigem == $servicoDestino)) {
                
                $composicoes = $this->entityManager->getRepository('\Teknisa\Model\Planejamento\Entities\Compcard')
                    ->findBy(
                        array(
                            "nrorg" => $this->environment->getNrOrg(),
                            "nrcardapio" => $nrCardapioOrigem,
                            "cdfilial" => $filialDestino
                        )
                    );
                    
                foreach ($composicoes as $composicao) {
                    $this->importarComposicao($montagem, $idcarddescentr, $idpratoespe, $composicao->getNrseqpratcar(), $composicao->getVrpeconscard(), $composicao->getCdprato(), $composicao->getCdfilauxprat(), $composicao->getNrordpratcar(), $composicao->getCdtipoprat(), $filialDestino);
                }

            // importar cardapio periodo diferente
            }
            else {
                $composicoes = $this->retornaComposicoesFormatadas($nrCardapioOrigem, $unidadeOrigem, $servicoDestino, $dataCardapioDestino, $filialDestino);
                
                if($composicoes) {

                    foreach ($composicoes as $composicao) {
                        $this->importarComposicao($montagem, $idcarddescentr, $idpratoespe, $composicao['nrseqcnfmes'], $composicao['vrpeconscard'], $composicao['cdprato'], $composicao['cdfilauxprat'], $composicao['nrordcnfmes'], $composicao['cdtipoprat'], $filialDestino);
                    }
                }
            }
        }
    }
    
    public function getIdAtivoPratosReceitasDia($row) {
        $query = "SELECT
                    P.CDPRATO,
                    P.CDFILAUXPRAT,
                    P.NMPRATO,
                    P.NRORG,
                    P.IDATIVO AS IDATIVOPRATO,
                    PC.IDATIVO AS IDATIVORECE
                FROM
                    CARDADIA CA,
                    COMPCARD CC,
                    PRATO P,
                    (
                        SELECT
                            CDPRATO,
                            CDFILAUXPRAT,
                            NRCORPRIPRAT AS NRCOR,
                            'P' AS TIPO,
                            IDATIVO
                        FROM
                            PRATPADR
                        UNION ALL
                        SELECT
                            CDPRATO,
                            CDFILAUXPRAT,
                            NRCORPRIFILI AS NRCOR,
                            'F' AS TIPO,
                            IDATIVO
                        FROM
                            PRATFILI
                        WHERE
                            CDFILIAL =:CDFILIAL
                        UNION ALL
                        SELECT
                            CDPRATO,
                            CDFILAUXPRAT,
                            NRCORPRISERV AS NRCOR,
                            'S' AS TIPO,
                            IDATIVO
                        FROM
                            PRATSERV
                        WHERE
                                CDFILIAL =:CDFILIAL
                            AND
                                CDSERVFILI =:CDSERVFILI
                    ) PC
                WHERE
                        CC.NRCARDAPIO = CA.NRCARDAPIO
                    AND
                        CC.CDPRATO = P.CDPRATO
                    AND
                        P.CDPRATO = PC.CDPRATO
                    AND
                        CA.CDSERVFILI = :CDSERVFILI
                    AND 
                        PC.CDFILAUXPRAT = CC.CDFILAUXPRAT
                    AND 
                        CC.CDFILIAL = CA.CDFILIAL
                    AND
                        CA.CDFILIAL = :CDFILIAL
                    AND
                        CA.DTCARDAPIO = :DTCARDAPIO
                    AND (
                            P.IDATIVO <> 'S'
                        OR
                            PC.IDATIVO <> 'S'
                    )
                GROUP BY
                    P.CDPRATO,
                    P.CDFILAUXPRAT,
                    P.NMPRATO,
                    P.NRORG,
                    P.IDATIVO,
                    PC.IDATIVO";
        $params = array(
            "CDFILIAL" => $row['CDFILIAL'],
            "CDSERVFILI" => $row['CDSERVFILI'],
            "DTCARDAPIO" => $row['DATA']
        );
        return $this->entityManager->getConnection()->fetchAll($query, $params);
    }

}