<?php
namespace Teknisa\PLA\Modules\Medicao\Services;

use \Teknisa\Model\Planejamento\Entities\Cardadia;
use \Teknisa\Model\Estoque\Entities\Lanctoestoq;
use \Estoque\Entities\Lanctoestoq as EstLanctoestoq;
use \Teknisa\Model\Planejamento\Entities\Retirada;
use \Teknisa\Model\Planejamento\Entities\Itemreti;
use \Teknisa\Model\Estoque\Entities\Itlanctoest;
use \Setforms2\Shared\ORM\DateTime;
use Teknisa\PebLibs\Services\ValidacoesEstoqueService;

class Pla02000RetiradaPlanejamentoService {

    protected $environment;
    protected $entityManager;
    protected $novoCodigo;
    protected $connection;
    protected $validacoesEstoqueService;
    const COD_ERROR_PEBBIAN = 1500;
	public function __construct($environment, $entityManager, $novoCodigo, ValidacoesEstoqueService $validacoesEstoqueService) {
        $this->environment = $environment;
        $this->entityManager = $entityManager;
		$this->connection = $entityManager->getConnection();
		$this->novoCodigo = $novoCodigo;
		$this->validacoesEstoqueService = $validacoesEstoqueService;
	}

    public function iniciaConexao() {
        $this->connection->beginTransaction();
    }

    public function realizaCommit() {
        if($this->connection->isTransactionActive())
            $this->connection->commit();
    }

    public function realizaRollback() {
        if($this->connection->isTransactionActive())
            $this->connection->rollback();
    }

    public function retornaNrcardapio($row) {
        $sql = "SELECT 
                    NRCARDAPIO 
                FROM CARDADIA 
                    WHERE CDFILIAL = :CDFILIAL 
                          AND CDSERVFILI = :CDSERVFILI
                          AND TRUNC(DTCARDAPIO) = :DTCARDAPIO
                          AND NRORG = :NRORG";
        $params = array(
            'CDFILIAL'     => $row['CDFILIAL'],
            'CDSERVFILI'   => $row['CDSERVFILI'],
            'DTCARDAPIO'   => $row['DTCARDAPIO'],
            'NRORG'        => $this->environment->getNrOrg()
        );
        $nrcardapio = $this->connection->fetchColumn($sql, $params);
        if (empty($nrcardapio)) {
            $nrcardapio = null;
        }
    	return $nrcardapio;
    }
    
    public function verificaParamExibeSaldo ($cdfilial) {
        $query= "SELECT PP.CDFILIAL, PP.IDEXIBESALDOESTRETI, PF.IDESTONLINE FROM PARAMPLANEJA PP, PARAMFILIAL PF WHERE PP.CDFILIAL = :P_CDFILIAL AND PF.CDFILIAL = PP.CDFILIAL";
        $params = array(
            "P_CDFILIAL" => $cdfilial
        );
        $result = $this->connection->fetchAll($query, $params);
        return $result;
        
    }

    public function insereCardapio($row) {
        $cardadia = new Cardadia();
        $nrorg = $this->environment->getNrOrg();
        $cdoperador = $this->environment->getCdOperador();
     	$nrcardapio = $this->novoCodigo->getCodeByTableName('CARDAPIO'.$row['CDFILIAL'], $nrorg);

        $cardadia->setNrcardapio($nrcardapio);
        $cardadia->setCdfilial($row['CDFILIAL']);
        $cardadia->setCdservfili($row['CDSERVFILI']);
        $cardadia->setDtcardapio($this->getDataDeString($row['DTCARDAPIO']));
        $cardadia->setDtmescard($this->getPrimeiroDiaDoMes($row['DTCARDAPIO']));
        $cardadia->setCdoperador($cdoperador);

        $cardadia->setNrorg($nrorg);
        $cardadia->setNrorginclusao($nrorg);
        $cardadia->setNrorgultatu($nrorg);
        $cardadia->setDtinclusao(new \DateTime());
        $cardadia->setDtultatu(new \DateTime());
        $cardadia->setCdoperinclusao($cdoperador);
        $cardadia->setCdoperultatu($cdoperador);

        $this->entityManager->persist($cardadia);
        $this->entityManager->flush();

        return $nrcardapio;
    }

    public function insereLancamento($row) {
        
        $lanctoestoq = new Lanctoestoq();
        $nrorg = $this->environment->getNrOrg();
        $cdoperador = $this->environment->getCdOperador();
        $nrlancestq = $this->novoCodigo->getCodeByTableName('LANCTOEST'.$this->environment->getCdFilial(), $nrorg);

        $lanctoestoq->setNrlancestq($nrlancestq);
        $lanctoestoq->setCdalmoxarife($row['CDALMOXARIFE']);
        $lanctoestoq->setCdlocalestoq($row['CDLOCALESTOQ']);
        $lanctoestoq->setCdfilial($this->environment->getCdFilial());
        $lanctoestoq->setCdfilimovi($row['CDFILIAL']);
        $lanctoestoq->setIdtplancto($row['IDTPLANCTO']);
        $lanctoestoq->setDtlancestq($this->getDataDeString($row['DTLANCESTQ']));
        $lanctoestoq->setDslancestq($row['DSLANCESTQ']);
        $lanctoestoq->setIdaltcusmed('N');
        $lanctoestoq->setIdstatusctbest('P');
        $lanctoestoq->setCdoperador($cdoperador);
        $lanctoestoq->setNrorg($nrorg);
        $lanctoestoq->setNrorginclusao($nrorg);
        $lanctoestoq->setNrorgultatu($nrorg);
        $lanctoestoq->setDtinclusao(new DateTime());
        $lanctoestoq->setDtultatu(new DateTime());
        $lanctoestoq->setCdoperinclusao($cdoperador);
        $lanctoestoq->setCdoperultatu($cdoperador);

        $this->entityManager->persist($lanctoestoq);
        $this->entityManager->flush();

        return $nrlancestq;
    }

    public function insereRetirada($row) {
        $retirada = new Retirada();
        $nrorg = $this->environment->getNrOrg();
        $cdoperador = $this->environment->getCdOperador();
        $nrretirada = $this->novoCodigo->getCodeByTableName('RETIRADA'.$row['CDFILIAL'], $nrorg);

        $retirada->setNrretirada($nrretirada);
        $retirada->setNrcardapio($row['NRCARDAPIO']);
        $retirada->setCdfilial($row['CDFILIAL']);
        $retirada->setNrorg($nrorg);
        $retirada->setDtretirada($this->getDataDeString($row['DTRETIRADA']));
        $retirada->setCdfilancestq($this->environment->getCdFilial());
        $retirada->setNrlancestq($row['NRLANCESTQ']);
        $retirada->setNrrequextrre($row['NRREQUEXTRRE']);
        $retirada->setIdretiprat($row['IDUTRETIPRAT']);

        $retirada->setNrorg($nrorg);
        $retirada->setNrorginclusao($nrorg);
        $retirada->setNrorgultatu($nrorg);
        $retirada->setDtinclusao(new \DateTime());
        $retirada->setDtultatu(new \DateTime());
        $retirada->setCdoperinclusao($cdoperador);
        $retirada->setCdoperultatu($cdoperador);

        $this->entityManager->persist($retirada);
        $this->entityManager->flush();

        return $nrretirada;
    }

    public function insereItemreti($row, $cdfilialFilterLancEst) {

        $nrorg = $this->environment->getNrOrg();
        $itemreti = new Itemreti();

        if (floatval(str_replace(',', '.', $row['QTRETIRADA'])) > 0) {
            $itemreti->setNrlancestq($row['NRLANCESTQ']);
            $itemreti->setNrsequitem($row['NRSEQUITEM']);
            $itemreti->setCdfiitlanest($cdfilialFilterLancEst);
        }

        $itemreti->setNrcardapio($row['NRCARDAPIO']);
        $itemreti->setNrretirada($row['NRRETIRADA']);
        $itemreti->setCdfilial($row['CDFILIAL']);
        $itemreti->setCdproduto($row['CDPRODUTO']);
        $itemreti->setCdtiporeti($row['CDTIPORETI']);
        $itemreti->setIdintesist($row['IDINTESIST']);
        $itemreti->setNrsequitreti($row['NRSEQUITEM']);
        $itemreti->setQtretirada($row['QTRETIRADA']);
        $itemreti->setQtprevisreti($row['QTPREVISRETI']);
        $itemreti->setQtextrareti($row['QTEXTRARETI']);
        $itemreti->setQtdevolvreti($row['QTDEVOLVRETI']);
        $itemreti->setCdprato($row['CDPRATO']);
        $itemreti->setCdfilauxprat($row['CDFILAUXPRAT']);
        $itemreti->setCdtipoprat($row['CDTIPOPRAT']);

        $itemreti->setNrorg($nrorg);
        $itemreti->setNrorginclusao($nrorg);
        $itemreti->setNrorgultatu($nrorg);
        $itemreti->setDtinclusao(new \DateTime());
        $itemreti->setDtultatu(new \DateTime());
        $itemreti->setCdoperinclusao($this->environment->getCdOperador());
        $itemreti->setCdoperultatu($this->environment->getCdOperador());

        $this->entityManager->persist($itemreti);
        $this->entityManager->flush();
        return true;
    }

    public function insereItemLancto($row, $filialLanc) {
        
        $nrorg = $this->environment->getNrOrg();
        $cdoperador = $this->environment->getCdOperador();
        if(floatval(str_replace(',', '.', $row['QTRETIRADA'])) > 0) {
            $itlanctoest = new Itlanctoest();

            $produto = $this->entityManager->getRepository('\Teknisa\Model\Geral\Entities\Produto')
                ->findOneBy(array('nrorg' => $nrorg, 'cdproduto' => $row['CDPRODUTO']));

            $vrfatoconv = $produto->getVrfatoconv();

            if($vrfatoconv && floatval(str_replace(',', '.', $vrfatoconv)) != 0) {
                $row['QTRETIRADA'] = $row['QTRETIRADA']*$vrfatoconv;
            }

            $prodmovi = $produto->getCdprodesto();
            if (!$prodmovi) {
                $prodmovi = $produto->getCdproduto();
            }

            $prodesto = $this->entityManager->getRepository('\Teknisa\Model\Geral\Entities\Produto')
                ->findOneBy(array('nrorg' => $nrorg, 'cdproduto' => $prodmovi));

            $paramfilial = $this->entityManager->getRepository('\Teknisa\Model\Geral\Entities\Paramfilial')
                ->findOneBy(array('nrorg' => $nrorg, 'cdfilial' => $row['CDFILIAL']));

            $tpcustoparamfilial = $paramfilial->getCdtipcustest();

            $vrunilancto = $this->consultaFuncaoCustoProdutoLCEST($row['CDFILIAL'], $row['CDPRODUTO'], $tpcustoparamfilial, $row['DTRETIRADA'],
                $row['CDALMOXARIFE'], $row['CDLOCALESTOQ'], $row['NRLOTEESTQ'], $row['NRSUBLOTE']);

            $vrtotlancto = floatval(str_replace(',','.',$vrunilancto))*$row['QTRETIRADA'];

            $itlanctoest->setCdfilial($filialLanc);
            $itlanctoest->setNrlancestq($row['NRLANCESTQ']);
            $itlanctoest->setNrsequitem($row['NRSEQUITEM']);
            $itlanctoest->setCdproduto($row['CDPRODUTO']);
            $itlanctoest->setQtlanctoest($row['QTRETIRADA']);
            $itlanctoest->setIdtipomovi($row['IDTIPOMOVI']);
            $itlanctoest->setDtlancmovi($this->getDataDeString($row['DTRETIRADA']));
            $itlanctoest->setIdordemclc($row['IDORDEMCLC']);
            $itlanctoest->setQttotlancto($row['QTRETIRADA']);
            $itlanctoest->setCdfilimovi($row['CDFILIAL']);
            $itlanctoest->setCdprodmovi($prodmovi);
            $itlanctoest->setIdevolestoq($row['IDEVOLESTOQ']);
            $itlanctoest->setCdalmoxarife($row['CDALMOXARIFE']);
            $itlanctoest->setCdlocalestoq($row['CDLOCALESTOQ']);
            $itlanctoest->setNrloteestq($row['NRLOTEESTQ']);
            $itlanctoest->setNrsublote($row['NRSUBLOTE']);
            $itlanctoest->setVrunilancto($vrunilancto);
            $itlanctoest->setVrtotlancto($vrtotlancto);
            $itlanctoest->setCdtiporeti($row['CDTIPORETI']);
            $itlanctoest->setCdoperador($cdoperador);

            $itlanctoest->setNrorg($nrorg);
            $itlanctoest->setNrorginclusao($nrorg);
            $itlanctoest->setNrorgultatu($nrorg);
            $itlanctoest->setDtinclusao(new \DateTime());
            $itlanctoest->setDtultatu(new \DateTime());
            $itlanctoest->setCdoperinclusao($cdoperador);
            $itlanctoest->setCdoperultatu($cdoperador);

            $this->entityManager->persist($itlanctoest);
            $this->entityManager->flush();
        }
        return true;
    }

    public function retornaMaxSequencial($row){
        $nrorg = $this->environment->getNrOrg();
        $itemreti = $this->entityManager->getRepository('\Teknisa\Model\Planejamento\Entities\Itemreti')
            ->findOneBy(array('nrorg' => $nrorg, 'cdfilial' => $row['CDFILIAL'], 'nrretirada' => $row['NRRETIRADA'], 'nrcardapio' => $row['NRCARDAPIO']), array('nrsequitreti' => 'DESC'));

        if ($itemreti) {
            $nrsequitreti = str_pad($itemreti->getNrsequitreti(), 4, "0", STR_PAD_LEFT);
        } else {
            $nrsequitreti = str_pad(0, 4, "0", STR_PAD_LEFT);
        }

        return $nrsequitreti;
    }

    public function verificaItemretiExiste($row) {
        $nrorg = $this->environment->getNrOrg();
        $itemreti = $this->entityManager->getRepository('\Teknisa\Model\Planejamento\Entities\Itemreti')
            ->findOneBy(array('nrorg' => $nrorg, 'cdfilial' => $row['CDFILIAL'], 'nrretirada' => $row['NRRETIRADA'], 'nrcardapio' => $row['NRCARDAPIO'],
            'cdtipoprat' => $row['CDTIPOPRAT'], 'cdprato' => $row['CDPRATO'], 'cdfilauxprat' => $row['CDFILAUXPRAT'], 'cdproduto' => $row['CDPRODUTO']));

        return $itemreti;
    }
    
    public function Vd_ExistProdAlmox($row) {
        $sql = <<<SQL
                SELECT A.CDPRODUTO
                  FROM ITEMRETI A
                       INNER JOIN ITLANCTOEST B ON B.CDFILIAL = A.CDFIITLANEST AND
                                                   B.NRLANCESTQ = A.NRLANCESTQ AND
                                                   B.NRSEQUITEM = A.NRSEQUITEM
                 WHERE (A.CDFILIAL   = :P_CDFILIAL)
                   AND (A.NRRETIRADA = :P_NRRETIRADA)
                   AND (A.CDPRODUTO = :P_CDPRODUTO)
                   AND (A.NRSEQUITRETI <> :P_NRSEQ)
                   AND (NVL(A.CDTIPOPRAT,' ')   = NVL(:P_CDTIPOPRAT,' '))
                   AND (NVL(A.CDPRATO,' ')      = NVL(:P_CDPRATO,' '))
                   AND (NVL(A.CDFILAUXPRAT,' ') = NVL(:P_CDFILAUXPRAT,' '))
                   AND (B.CDALMOXARIFE = :P_CDALMOXARIFE)
                   AND (B.CDLOCALESTOQ = :P_CDLOCALESTOQ)
                   AND (B.NRLOTEESTQ   = :P_NRLOTEESTQ)
                   AND (B.NRSUBLOTE    = :P_NRSUBLOTE)
SQL;
        $params = array(
            'P_CDFILIAL' => $row['CDFILIAL'],
            'P_NRRETIRADA' => $row['NRRETIRADA'],
            'P_CDPRODUTO' => $row['CDPRODUTO'],
            'P_NRSEQ' => $row['NRSEQUITRETI'],
            'P_CDTIPOPRAT' => isset($row['CDTIPOPRAT']) && $row['CDTIPOPRAT'] ? $row['CDTIPOPRAT'] : ' ',
            'P_CDPRATO'      => isset($row['CDPRATO'])      && $row['CDPRATO']      ? $row['CDPRATO']      : ' ',
            'P_CDFILAUXPRAT' => isset($row['CDFILAUXPRAT']) && $row['CDFILAUXPRAT'] ? $row['CDFILAUXPRAT'] : ' ',
            'P_CDALMOXARIFE' => isset($row['CDALMOXARIFE']) && $row['CDALMOXARIFE'] ? $row['CDALMOXARIFE'] : ' ',
            'P_CDLOCALESTOQ' => isset($row['CDLOCALESTOQ']) && $row['CDLOCALESTOQ'] ? $row['CDLOCALESTOQ'] : ' ',
            'P_NRLOTEESTQ'   => isset($row['NRLOTEESTQ'])   && $row['NRLOTEESTQ']   ? $row['NRLOTEESTQ']   : ' ',
            'P_NRSUBLOTE'    => isset($row['NRSUBLOTE'])    && $row['NRSUBLOTE']    ? $row['NRSUBLOTE']    : ' '
        );

        try {
            return $this->connection->fetchColumn($sql, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }
    

    public function retornaItlanctoest($cdfilial, $nrlancestq, $nrsequitem) {
        $nrorg = $this->environment->getNrOrg();
        $itlanctoest = $this->entityManager->getRepository('Estoque\Entities\Itlanctoest')
            ->findOneBy(array('nrorg' => $nrorg, 'cdfilial' => $cdfilial, 'nrlancestq' => $nrlancestq,
            'nrsequitem' => $nrsequitem));

        return $itlanctoest;
    }

    public function alteraItemLancto($row) {
        $nrorg = $this->environment->getNrOrg();
        $itlanctoest = $this->retornaItlanctoest($row['CDFILIAL'], $row['NRLANCESTQ'], $row['NRSEQUITRETI']);

        if($itlanctoest) {
            $itlanctoest->setQtlanctoest($row['QTRETIRADA']);
            $itlanctoest->setQttotlancto($row['QTRETIRADA']);
            $itlanctoest->setCdtiporeti($row['CDTIPORETI']);
            $itlanctoest->setVrtotlancto($itlanctoest->getVrunilancto() * $row['QTRETIRADA']);
            //Campos do estoque
            $itlanctoest->setCdalmoxarife($row['CDALMOXARIFE']);
            $itlanctoest->setCdlocalestoq($row['CDLOCALESTOQ']);
            $itlanctoest->setNrloteestq($row['NRLOTEESTQ']);
            $itlanctoest->setNrsublote($row['NRSUBLOTE']);
            
            $itlanctoest->setNrorgultatu($nrorg);
            $itlanctoest->setDtultatu(new \DateTime());
            $itlanctoest->setCdoperultatu($this->environment->getCdOperador());

            $this->entityManager->persist($itlanctoest);
            $this->entityManager->flush();
        }
        return true;
    }

    public function removeRelacaoItemRetiItlanctoest($row) {
        $itemreti = $this->retornaItemreti($row);
        if ($itemreti) {
            $itemreti->setCdfiitlanest(null);
            $itemreti->setNrlancestq(null);
            $itemreti->setNrsequitem(null);
            $this->entityManager->persist($itemreti);
            $this->entityManager->flush();
        }
    }

    public function excluiItemLancto($row) {
        $itlanctoest = $this->retornaItlanctoest($row['CDFILANCESTQ'], $row['NRLANCESTQ'], $row['NRSEQUITRETI']);
        if($itlanctoest) {
            $this->entityManager->remove($itlanctoest);
            $this->entityManager->flush();
        }
        return true;
    }

    public function retornaItemreti($row) {
        $nrorg = $this->environment->getNrOrg();
        $itemreti = $this->entityManager->getRepository('\Teknisa\Model\Planejamento\Entities\Itemreti')
            ->findOneBy(array('nrorg' => $nrorg, 'cdfilial' => $row['CDFILIAL'], 'nrretirada' => $row['NRRETIRADA'],
            'nrcardapio' => $row['NRCARDAPIO'], 'nrsequitreti' => $row['NRSEQUITRETI']));

        return $itemreti;
    }

    public function alteraItemreti($row) {
        $nrorg = $this->environment->getNrOrg();
        $itemreti = $this->retornaItemreti($row);

        if($itemreti) {
            if(floatval(str_replace(',', '.', $row['QTRETIRADA'])) > 0) {
                $itemreti->setNrlancestq($row['NRLANCESTQ']);
                $itemreti->setNrsequitem($row['NRSEQUITRETI']);
                $itemreti->setCdfiitlanest($row['CDFILANCESTQ']);
            }

            $itemreti->setQtretirada($row['QTRETIRADA']);
            $itemreti->setQtextrareti($row['QTEXTRARETI']);
            $itemreti->setQtdevolvreti($row['QTDEVOLVRETI']);
            $itemreti->setCdtiporeti($row['CDTIPORETI']);

            $itemreti->setNrorgultatu($nrorg);
            $itemreti->setDtultatu(new \DateTime());
            $itemreti->setCdoperultatu($this->environment->getCdOperador());

            $this->entityManager->persist($itemreti);
            $this->entityManager->flush();
        }

        return true;
    }

    public function excluiItemreti($row) {
        $itemreti = $this->retornaItemreti($row);

        if($itemreti) {
            $this->entityManager->remove($itemreti);
            $this->entityManager->flush();
        }

        return true;
    }

    public function retornaItensPorPrato($row) {
        $nrorg = $this->environment->getNrOrg();
        $itens = $this->entityManager->getRepository('\Teknisa\Model\Planejamento\Entities\Itemreti')
            ->findBy(array('nrorg' => $nrorg, 'cdfilial' => $row['CDFILIAL'], 'nrretirada' => $row['NRRETIRADA'],
            'nrcardapio' => $row['NRCARDAPIO'], 'cdtipoprat' => $row['CDTIPOPRAT'], 'cdprato' => $row['CDPRATO'], 'cdfilauxprat' => $row['CDFILAUXPRAT']));

        return $itens;
    }
    
    // codigo definido para exceções em geral na atualizar estoque pebbian: 1500
    public function getCodErrorPebbian(){
        return self::COD_ERROR_PEBBIAN;
    }
    
    public function atualizarEstoque($cdfilialLancamento, $nrlancestq, $abortaOperacao = true) {
        try {
            $this->validacoesEstoqueService->atualizaEstoque($cdfilialLancamento, $nrlancestq, false, true, $abortaOperacao);
        } catch(\Exception $e){
            throw new \Exception($e->getMessage(), self::COD_ERROR_PEBBIAN);
        }
    }
    public function excluiLancamentoRetirada($row) {
        $this->iniciaConexao();
        try {
            $nrorg = $this->environment->getNrOrg();
            $cdfilial = $row['CDFILIAL'];
            $nrretirada = $row['NRRETIRADA'];
            $nrcardapio = $row['NRCARDAPIO'];
            $nrlancestq = $row['NRLANCESTQ'];
            $cdfilancestq = $this->retornaFilialLancamentoRetirada($cdfilial, $nrcardapio);
            $cardadia = $this->entityManager->getRepository('\Teknisa\Model\Planejamento\Entities\Cardadia')
                ->findOneBy(array('nrorg' => $nrorg, 'cdfilial' => $cdfilial, 'nrcardapio' => $nrcardapio));
    
            $dtcardapio = $cardadia->getDtcardapio();
            $cdservfili = $cardadia->getCdservfili();
    
            $dsparamfatfi = $this->retornaDsparamfatfi($cdfilial);
    
            if($dsparamfatfi == 'S') {
                $nrapuracfat = $this->retornaNrapuracfat($cdfilial, $cdservfili, $dtcardapio->format('d/m/Y'));
    
                if($nrapuracfat) {
                    throw new \Exception("Existem efetivos faturados. Exclusão bloqueada!");
                }
            }
            
            $this->excluiItensRetirada($cdfilial, $nrretirada, $nrcardapio, false);
    
            $this->excluiItensEstoque($cdfilancestq, $nrlancestq);
    
            $this->validacoesEstoqueService->atualizaEstoque($cdfilancestq, $nrlancestq);
            
            $this->excluiRetirada($cdfilial, $nrretirada, $nrcardapio);
    
            
            $this->excluiLancamento($cdfilancestq, $nrlancestq);
    
            $this->limpaComensaisCompcard($cdfilial, $nrcardapio);
    
            $this->limpaComensaisEfetclie($cdfilial, $cdservfili, $dtcardapio);
            
            $this->realizaCommit();
        } catch (\Exception $e) {
            $this->realizaRollback();
            throw $e;
        }
    }

    public function retornaLanctoestoq($cdfilial, $nrlancestq, $filterNameFilial = 'cdfilial') {
        $nrorg = $this->environment->getNrOrg();
        $filter = array(
            'nrorg' => $nrorg,
            'nrlancestq' => $nrlancestq
        );
        $filter[$filterNameFilial] = $cdfilial;
        $lanctoestoq = $this->entityManager->getRepository('\Teknisa\Model\Estoque\Entities\Lanctoestoq')
            ->findOneBy($filter);

        return $lanctoestoq;
    }

    public function retornaItensEstoque($cdfilial, $nrlancestq) {
        $nrorg = $this->environment->getNrOrg();
        $itensestoque = $this->entityManager->getRepository('\Teknisa\Model\Estoque\Entities\Itlanctoest')
            ->findBy(array('nrorg' => $nrorg, 'cdfilial' => $cdfilial, 'nrlancestq' => $nrlancestq));
        return $itensestoque;
    }

    public function retornaProdutosMovilote($cdfilimovi, $nrlancestq, $cdfililanc) {
        $nrorg = $this->environment->getNrOrg();
        $produtosirregulares = $this->entityManager->getRepository('\Teknisa\Model\Estoque\Entities\Movilote')
            ->findBy(array('nrorg' => $nrorg, 'nrmovilote' => $cdfililanc.$nrlancestq, 'cdfilial' => $cdfilimovi));

        return $produtosirregulares;
    }

    public function retornaProdutoEstoque($cdfilial, $nrlancestq, $cdproduto) {
        $nrorg = $this->environment->getNrOrg();
        $produtosestoque = $this->entityManager->getRepository('\Teknisa\Model\Estoque\Entities\Itlanctoest')
            ->findOneBy(array('nrorg' => $nrorg, 'cdfilial' => $cdfilial, 'nrlancestq' => $nrlancestq, 'cdproduto' => $cdproduto));

        return $produtosestoque;
    }

    public function retornaItensRetirada( $cdfilial, $nrretirada, $nrcardapio) {
        $nrorg = $this->environment->getNrOrg();
        $itensestoque = $this->entityManager->getRepository('\Teknisa\Model\Planejamento\Entities\Itemreti')
            ->findBy(array('nrorg' => $nrorg, 'cdfilial' => $cdfilial, 'nrretirada' => $nrretirada, 'nrcardapio' => $nrcardapio));

        return $itensestoque;
    }

    public function excluiItensEstoque($cdfilial, $nrlancestq, $isControlTransaction = true) {
        $itensestoque = $this->retornaItensEstoque($cdfilial, $nrlancestq);

        if($isControlTransaction)
            $this->iniciaConexao();

        foreach ($itensestoque as $itlanctoest) {
            $this->entityManager->remove($itlanctoest);
            $this->entityManager->flush();
        }
        if($isControlTransaction)
            $this->realizaCommit();
    }

    public function excluiItensRetirada($cdfilial, $nrretirada, $nrcardapio, $isControlTransaction = true) {
        $nrorg = $this->environment->getNrOrg();
        $itensretirada = $this->retornaItensRetirada($cdfilial, $nrretirada, $nrcardapio);
        if($isControlTransaction)
            $this->iniciaConexao();

        foreach ($itensretirada as $itemreti) {
            $this->entityManager->remove($itemreti);
            $this->entityManager->flush();
        }
        if($isControlTransaction)
            $this->realizaCommit();
    }

    public function excluiLancamento($cdfilial, $nrlancestq) {
        $lanctoestoq = $this->retornaLanctoestoq($cdfilial, $nrlancestq);

        if($lanctoestoq) {
            $this->entityManager->remove($lanctoestoq);
            $this->entityManager->flush();
        }

        return true;
    }

    public function excluiRetirada($cdfilial, $nrretirada, $nrcardapio) {
        $retirada = $this->retornaRetirada($cdfilial, $nrretirada, $nrcardapio);

        if($retirada) {
            $this->entityManager->remove($retirada);
            $this->entityManager->flush();
        }

        return true;
    }

    public function retornaRetirada($cdfilial, $nrretirada, $nrcardapio) {
        $nrorg = $this->environment->getNrOrg();
        $retirada = $this->entityManager->getRepository('\Teknisa\Model\Planejamento\Entities\Retirada')
            ->findOneBy(array('nrorg' => $nrorg, 'cdfilial' => $cdfilial, 'nrretirada' => $nrretirada, 'nrcardapio' => $nrcardapio));

        return $retirada;
    }

    public function limpaComensaisCompcard($cdfilial, $nrcardapio) {
        $nrorg = $this->environment->getNrOrg();
        $compcard = $this->entityManager->getRepository('\Teknisa\Model\Planejamento\Entities\Compcard')
            ->findOneBy(array('nrorg' => $nrorg, 'cdfilial' => $cdfilial, 'nrcardapio' => $nrcardapio));

        if($compcard){
            $compcard->setQtcomefetpra(null);
            $compcard->setQtproduzprat(null);
            $this->entityManager->flush();
        }

        return true;
    }

    public function limpaComensaisEfetclie($cdfilial, $cdservfili, $dtefetclie) {
        $nrorg = $this->environment->getNrOrg();
        $efetclie = $this->entityManager->getRepository('Faturamento\Entities\Efetclie')
            ->findOneBy(array('nrorg' => $nrorg, 'cdfilial' => $cdfilial, 'cdservfili' => $cdservfili, 'dtefetclie' => $dtefetclie));

        if($efetclie){
            $efetclie->setQtcomefetcli(null);
            $efetclie->setQtdoaefetcli(null);
            $efetclie->setQtfunefetcli(null);
            $this->entityManager->flush();
        }

        return true;
    } 


    /*
    * retorna uma mensagem de erro se a quantidade de estoque for invalida
    * ou false se for valida
    */
    public function validaQuantidadeEstoque($row) {
        $nrorg = $this->environment->getNrOrg();
        $nvlcdprod = isset($row['CDARVPROD']) && $row['CDARVPROD'] ? $row['CDARVPROD'] : $row['CDPRODUTO'];
        $cdfililanc = isset($row['CDFILILANC']) ? $row['CDFILILANC'] : null;
        $cdprodmovi = isset($row['CDPRODMOVI']) ? $row['CDPRODMOVI'] : null;
        if (!$cdfililanc) {
            $lanctoestoq = $this->retornaLanctoestoq($row['CDFILIAL'], $row['NRLANCESTQ']);
            if ($lanctoestoq) {
                $cdfililanc = $lanctoestoq->getCdFilial();
            } else {
                $cdfililanc = $this->environment->getCdFilial();
            }
        }

        if(!$cdprodmovi) {
            $produto = $this->entityManager->getRepository('\Teknisa\Model\Geral\Entities\Produto')
                ->findOneBy(array('nrorg' => $nrorg, 'cdproduto' => $row['CDPRODUTO']));
            $cdprodmovi = $produto->getCdprodesto();
            if (!$cdprodmovi) {
                $cdprodmovi = $produto->getCdproduto();
            }
        }

        if($row['QTRETIRADA'] < 0) {
            return 'Quantidade deve ser maior do que zero.';
        } else {
            $paramfilial = $this->entityManager->getRepository('\Teknisa\Model\Geral\Entities\Paramfilial')
                ->findOneBy(array('nrorg' => $nrorg, 'cdfilial' => $row['CDFILIAL']));

            $idestonline = $paramfilial->getIdestonline();
            $idpermvisposest = $paramfilial->getIdpermvisposest();
            if($idestonline != 'N') {
                $quantidadeEstoque = $this->verificaQuantidadeEstoque($row['CDFILIAL'], $cdfililanc, $row['NRLANCESTQ'], $row['NRSEQUITEM'], $row['CDPRODUTO'],
                    $cdprodmovi, $row['QTRETIRADA'], $row['DTPOSICAO'], $row['CDALMOXARIFE'], $row['CDLOCALESTOQ'], $row['NRLOTEESTQ'], $row['NRSUBLOTE']);
                if(!$quantidadeEstoque) {
                    return 'Produto não possui movimentação de estoque.';
                } else if(floatval(str_replace(',', '.', $quantidadeEstoque[0]['QTNEG'])) < 0) {
                    if(floatval(str_replace(',', '.', $row['QTRETIRADA'])) > floatval(str_replace(',', '.', $quantidadeEstoque[0]['QTESTOQDIA']))) {
                        if($row['IDEXIBESALDOESTRETI'] == 'S') {
                            return 'A quantidade consumida informada para o produto é maior do que o saldo (' .number_format(floatval(str_replace(',', '.', $quantidadeEstoque[0]['QTESTOQDIA'])),5,",",".").' '.$quantidadeEstoque[0]['SGUNIDADE']. ') disponível no estoque.';
                        } else {
                            return 'Quantidade informada não disponível no estoque.';
                        }
                    }
                }

                $arredondParametrizado = $this->retornaArredondamentoParametrizado($row['CDFILIAL'], $row['CDPRODUTO']);

                if (floatval(str_replace(',', '.', $arredondParametrizado[0]['QTESTMINSAID'])) > 0) {
                    $doValor = floatval(str_replace(',', '.', $row['QTRETIRADA'])) / floatval(str_replace(',', '.', $arredondParametrizado[0]['QTESTMINSAID']));
                    $doValorInt = round(floatval(str_replace(',', '.', $row['QTRETIRADA'])) / floatval(str_replace(',', '.', $arredondParametrizado[0]['QTESTMINSAID'])));
                    if (($doValor - $doValorInt) <> 0) {
                        return 'Quantidade informada deve ser múltiplo do arredondamento parametrizado para a saída: '+$arredondParametrizado[0]['QTESTMINSAID']+' '+$arredondParametrizado[0]['SGUNIDADE'];
                    }
                }

                $quantidadeMinimaProduto = $this->retornaQuantidadeMinimaProdutoEstoque($row['CDFILIAL'], $cdprodmovi);
                if(!$quantidadeMinimaProduto || floatval(str_replace(',', '.', $quantidadeMinimaProduto)) < 0) {
                    $quantidadeMinimaProduto = -1;
                }

                if(floatval(str_replace(',', '.', $quantidadeMinimaProduto)) > 0) {
                    if ((floatval(str_replace(',', '.', $quantidadeEstoque[0]['QTESTOQDIA'])) - floatval(str_replace(',', '.', $row['QTRETIRADA']))) < floatval(str_replace(',', '.', $quantidadeMinimaProduto))) {
                        return 'Aviso: a quantidade do produto no estoque ficará abaixo da quantidade mínima estipulada.';
                    }
                }
            }
        }

        return false;
    }

    public function getDataDeString($data) {
        $explode = explode('/', $data);
        $dateFormat = new DateTime($explode[2].'-'.$explode[1].'-'.$explode[0]);
        $dateFormat->setTime(0,0);
        return $dateFormat;
    }

    public function getPrimeiroDiaDoMes($data) {
        $explode = explode('/', $data);
        $firstDateFormat = new DateTime($explode[2].'-'.$explode[1].'-01');
        $firstDateFormat->setTime(0,0);
        return $firstDateFormat;
    }

    public function consultaFuncaoCustoProdutoLCEST($cdfilial, $cdproduto, $tpcustoparamfilial, $dtlancto, $cdalmox, $cdlocal, $nrlote, $nrsublote) {
        $sql = <<<SQL
        SELECT
          CUSTO_PRODUTO_LCEST(:CDFILIAL, :CDPRODUTO, TO_NUMBER(NVL(:CUSTO, '3')),
          TO_DATE(:DTPOSICAO), SYSDATE, :CDALMOXARIFE, :CDLOCALESTOQ,
          :NRLOTEESTQ, :NRSUBLOTE) AS CUSTO
        FROM
          DUAL
SQL;

        $params = array(
            'CDFILIAL'     => $cdfilial,
            'CDPRODUTO'    => $cdproduto,
            'CUSTO'        => $tpcustoparamfilial,
            'DTPOSICAO'    => $dtlancto,
            'CDALMOXARIFE' => $cdalmox,
            'CDLOCALESTOQ' => $cdlocal,
            'NRLOTEESTQ'   => $nrlote,
            'NRSUBLOTE'    => $nrsublote,
        );

        try {
            return $this->connection->fetchColumn($sql, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function retornaProdutosIrregulares($cdfilimovi, $nrcardapio, $nrretirada, $nrlancestq, $cdfililanc) {
        $sql = <<<SQL
        SELECT
          ITEMRETI.NRORG,
          ITEMRETI.CDFILIAL,
          ITEMRETI.NRRETIRADA,
          ITEMRETI.NRCARDAPIO,
          ITEMRETI.CDTIPOPRAT,
          TIPOPRAT.NMTIPOPRAT,
          ITEMRETI.CDPRATO,
          PRATO.NMPRATO,
          PRODUTO.CDARVPROD,
          ITEMRETI.CDPRODUTO,
          PRODUTO.NMPRODUTO,
          PRODUTO.SGUNIDADE,
          ROUND(ITEMRETI.QTPREVISRETI, 5) QTPREVISRETI,
          ROUND(ITEMRETI.QTEXTRARETI, 5) QTEXTRARETI,
          ROUND(ITEMRETI.QTDEVOLVRETI, 5) QTDEVOLVRETI,
          ROUND(ITEMRETI.QTRETIRADA, 5) QTRETIRADA,
          ITEMRETI.NRSEQUITRETI,
          ITEMRETI.CDFILAUXPRAT,
          ITEMRETI.CDTIPORETI,
          ITEMRETI.CDFIITLANEST,
          ITLANCTOEST.NRSEQUITEM,
          ITLANCTOEST.CDPRODMOVI,
          ITLANCTOEST.DTLANCMOVI,
          ITLANCTOEST.CDALMOXARIFE,
          ITLANCTOEST.CDLOCALESTOQ,
          ITLANCTOEST.NRLOTEESTQ,
          ITLANCTOEST.NRSUBLOTE
        FROM
          PRODUTO,
          RETIRADA,
          ITEMRETI,
          MOVILOTE,
          TIPOPRAT,
          PRATO,
          ITLANCTOEST
        WHERE
          ITEMRETI.NRORG          = RETIRADA.NRORG
        AND ITEMRETI.CDFILIAL     = RETIRADA.CDFILIAL
        AND ITEMRETI.NRCARDAPIO   = RETIRADA.NRCARDAPIO
        AND ITEMRETI.NRRETIRADA   = RETIRADA.NRRETIRADA
        AND ITEMRETI.NRORG        = PRODUTO.NRORG
        AND ITEMRETI.CDPRODUTO    = PRODUTO.CDPRODUTO
        AND ITEMRETI.NRORG        = TIPOPRAT.NRORG(+)
        AND ITEMRETI.CDTIPOPRAT   = TIPOPRAT.CDTIPOPRAT(+)
        AND ITEMRETI.NRORG        = PRATO.NRORG(+)
        AND ITEMRETI.CDPRATO      = PRATO.CDPRATO(+)
        AND ITEMRETI.CDFILAUXPRAT = PRATO.CDFILAUXPRAT(+)
        AND ITEMRETI.NRORG        = ITLANCTOEST.NRORG(+)
        AND ITEMRETI.CDFIITLANEST = ITLANCTOEST.CDFILIAL(+)
        AND ITEMRETI.NRLANCESTQ   = ITLANCTOEST.NRLANCESTQ(+)
        AND ITEMRETI.NRSEQUITEM   = ITLANCTOEST.NRSEQUITEM(+)
        AND ITEMRETI.NRORG        = MOVILOTE.NRORG
        AND ITEMRETI.CDFILIAL     = MOVILOTE.CDFILIAL
        AND ITEMRETI.CDPRODUTO    = MOVILOTE.CDPRODUTO
        AND ITEMRETI.NRORG        = :NRORG
        AND ITEMRETI.CDFILIAL     = :CDFILIAL
        AND ITEMRETI.NRCARDAPIO   = :NRCARDAPIO
        AND ITEMRETI.NRRETIRADA   = :NRRETIRADA
        AND MOVILOTE.NRMOVILOTE   = :NRMOVILOTE
        ORDER BY
          ITEMRETI.NRSEQUITRETI
SQL;

        $params = array(
            'NRORG'     => $this->environment->getNrOrg(),
            'CDFILIAL'     => $cdfilimovi,
            'NRCARDAPIO'     => $nrcardapio,
            'NRRETIRADA'     => $nrretirada,
            'NRMOVILOTE'    => $cdfililanc.$nrlancestq
        );

        try {
            return $this->connection->fetchAll($sql, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function retornaDsparamfatfi($cdfilial) {
        $sql = <<<SQL
        SELECT
          SUBSTR(DSPARAMFATFI, 15, 1) AS DSPARAMFATFI
        FROM
          PARAMFILIAL
        WHERE
          NRORG      = :NRORG
        AND CDFILIAL = :CDFILIAL
SQL;

        $params = array(
            'NRORG'    => $this->environment->getNrOrg(),
            'CDFILIAL' => $cdfilial
        );

        try {
            return $this->connection->fetchColumn($sql, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function retornaNrapuracfat($cdfilial, $cdservfili, $dtefetclie) {
        $sql = <<<SQL
        SELECT
          NRAPURACFAT
        FROM
          EFETCLIE
        WHERE
          NRORG          = :NRORG
        AND CDFILIAL     = :CDFILIAL
        AND CDSERVFILI   = :CDSERVFILI
        AND DTEFETCLIE   = :DTEFETCLIE
        AND NRAPURACFAT IS NOT NULL
        AND
          (
            QTCOMEFETCLI  IS NOT NULL
          OR QTDOAEFETCLI IS NOT NULL
          OR QTFUNEFETCLI IS NOT NULL
          )
SQL;

        $params = array(
            'NRORG'      => $this->environment->getNrOrg(),
            'CDFILIAL'   => $cdfilial,
            'CDSERVFILI' => $cdservfili,
            'DTEFETCLIE' => $dtefetclie
        );

        try {
            return $this->connection->fetchColumn($sql, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function verificaQuantidadeEstoque($cdfilial, $cdfililanc, $nrlancestq, $nrsequitem, $cdproduto, $cdprodmovi, $qttotlancto, $dtposicao, $cdalmoxarife, $cdlocalestoq, $nrloteestq, $nrsublote) {
      $sql = <<<SQL
        SELECT
          P.CDARVPROD,
          P.CDPRODUTO,
          P.NMPRODUTO,
          P.SGUNIDADE,
          P.VRFATOCONV,
          (D.QTESTOQDIA + NVL(C.QTLANCTOEST,0)/P.VRFATOCONV) AS QTESTOQDIA,
          TO_CHAR((D.QTESTOQDIA+NVL(C.QTLANCTOEST,0)/P.VRFATOCONV) - :QTTOTLANCTO) AS QTNEG
        FROM POSESTDIA D,
           PRODUTO P
           LEFT JOIN (SELECT NRORG, CDPRODUTO, CDPRODMOVI, SUM(QTLANCTOEST) AS QTLANCTOEST
                      FROM ITLANCTOEST
                      WHERE (NRORG      = :NRORG)      AND
                            (CDFILIAL   = :CDFILILANC) AND
                            (NRLANCESTQ = :NRLANCESTQ) AND
                            (NRSEQUITEM = :NRSEQUITEM) AND
                            (CDPRODUTO  = :CDPRODUTO)  AND
                            (NVL(IDEVOLESTOQ, 'S') = 'S')
                      GROUP BY NRORG, CDPRODUTO, CDPRODMOVI) C
           ON
              (P.NRORG    = C.NRORG) AND
              (P.CDPRODUTO    = C.CDPRODMOVI)
        WHERE (P.CDPRODUTO    = :CDPRODMOVI)   AND
              (D.NRORG        = :NRORG)        AND
              (D.CDFILIAL     = :CDFILIAL)     AND
              (D.CDALMOXARIFE = :CDALMOXARIFE) AND
              (D.CDLOCALESTOQ = :CDLOCALESTOQ) AND
              (D.NRLOTEESTQ   = :NRLOTEESTQ)   AND
              (D.NRSUBLOTE    = :NRSUBLOTE)    AND
              (P.NRORG        = D.NRORG)       AND
              (P.CDPRODESTO   = D.CDPRODUTO)   AND
              ((D.DTPOSIESTQ  = (SELECT MAX(DTPOSIESTQ)
                               FROM POSESTDIA
                               WHERE (NRORG        = D.NRORG)        AND
                                     (CDFILIAL     = D.CDFILIAL)     AND
                                     (CDALMOXARIFE = D.CDALMOXARIFE) AND
                                     (CDLOCALESTOQ = D.CDLOCALESTOQ) AND
                                     (NRLOTEESTQ   = D.NRLOTEESTQ)   AND
                                     (NRSUBLOTE    = D.NRSUBLOTE)    AND
                                     (CDPRODUTO    = P.CDPRODESTO)   AND
                                     (DTPOSIESTQ  <= TO_DATE(:DTPOSICAO)))) OR (DTPOSIESTQ IS NULL))
        ORDER BY
           P.CDARVPROD
SQL;

      $params = array(
          'NRORG' => $this->environment->getNrOrg(),
          'CDFILIAL' => $cdfilial,
          'CDFILILANC' => $cdfililanc,
          'NRLANCESTQ' => $nrlancestq,
          'NRSEQUITEM' => $nrsequitem,
          'CDPRODUTO' => $cdproduto,
          'CDPRODMOVI' => $cdprodmovi,
          'QTTOTLANCTO' => $qttotlancto,
          'DTPOSICAO' => $dtposicao,
          'CDALMOXARIFE' => $cdalmoxarife,
          'CDLOCALESTOQ' => $cdlocalestoq,
          'NRLOTEESTQ' => $nrloteestq,
          'NRSUBLOTE' => $nrsublote
      );
      try {
            return $this->connection->fetchAll($sql, $params);
      } catch (\Exception $e) {
          throw $e;
      }
    }

    public function retornaArredondamentoParametrizado($cdfilial, $cdproduto) {
      $sql = <<<SQL
        SELECT
          PF.CDFILIAL,
          PF.CDPRODUTO,
          RTRIM(PF.CDALMOXARIFE)           AS CDALMOXARIFE,
          RTRIM(PF.CDLOCALESTOQ)           AS CDLOCALESTOQ,
          (PF.QTESTMINSAID / P.VRFATOCONV) AS QTESTMINSAID,
          P.SGUNIDADE,
          IDUTCTRLLOTE,
          NVL(PF.IDCNTRESTOQ, 'S') AS IDCNTRESTOQ,
          NVL(IDCTRLESTQ, 'S')     AS IDCTRLESTQ
        FROM
          PRODFILI PF,
          PRODUTO P,
          PARAMFILIAL F
        WHERE
          PF.NRORG       = :NRORG
        AND PF.CDFILIAL  = :CDFILIAL
        AND P.CDPRODUTO  = :CDPRODUTO
        AND PF.NRORG     = P.NRORG
        AND PF.CDPRODUTO = P.CDPRODESTO
        AND PF.CDFILIAL  = F.CDFILIAL
        AND PF.NRORG     = F.NRORG
SQL;

      $params = array(
          'NRORG' => $this->environment->getNrOrg(),
          'CDFILIAL' => $cdfilial,
          'CDPRODUTO' => $cdproduto
      );

      try {
          return $this->connection->fetchAll($sql, $params);
      } catch (\Exception $e) {
          throw $e;
      }
    }

    public function retornaQuantidadeMinimaProdutoEstoque($cdfilial, $cdproduto) {
      $sql = <<<SQL
        SELECT
          PRODFILI.QTESTOQMIN
        FROM
          PRODFILI,
          PRODUTO
        WHERE
          PRODUTO.NRORG           = :NRORG
        AND PRODUTO.CDPRODUTO     = :CDPRODUTO
        AND PRODFILI.NRORG (+)    = PRODUTO.NRORG
        AND PRODFILI.CDFILIAL (+) = :CDFILIAL
        AND PRODFILI.CDPRODUTO(+) = PRODUTO.CDPRODUTO
SQL;

      $params = array(
          'NRORG' => $this->environment->getNrOrg(),
          'CDFILIAL' => $cdfilial,
          'CDPRODUTO' => $cdproduto
      );

      try {
          return $this->connection->fetchColumn($sql, $params);
      } catch (\Exception $e) {
          throw $e;
      }
    }

    public function retornaRetiradasExistentes($cdfilial, $nrcardapio, $nrrequextrre) {
      $sql = <<<SQL
        SELECT RETIRADA.NRRETIRADA AS "NRRETIRADA"
        FROM RETIRADA
        WHERE RETIRADA.CDFILIAL = :CDFILIAL
        AND RETIRADA.NRCARDAPIO = :NRCARDAPIO
        AND RETIRADA.NRREQUEXTRRE = :NRREQUEXTRRE
SQL;

      $params = array(
          'NRORG' => $this->environment->getNrOrg(),
          'CDFILIAL' => $cdfilial,
          'NRCARDAPIO' => $nrcardapio,
          'NRREQUEXTRRE' => $nrrequextrre
      );

      try {
          return $this->connection->fetchAll($sql, $params);
      } catch (\Exception $e) {
          throw $e;
      }
    }
    
    public function retornaFilialLancamentoRetirada($cdfilial, $nrcardapio){
        $sql = "SELECT RETIRADA.CDFILANCESTQ
                FROM CARDADIA 
                    JOIN RETIRADA
                        ON CARDADIA.NRORG = RETIRADA.NRORG
                            AND CARDADIA.CDFILIAL = RETIRADA.CDFILIAL
                            AND CARDADIA.NRCARDAPIO = RETIRADA.NRCARDAPIO
                WHERE 
                        RETIRADA.NRORG = :NRORG
                    AND RETIRADA.CDFILIAL = :CDFILIAL
                    AND RETIRADA.NRCARDAPIO = :NRCARDAPIO";
      $params = array(
          'NRORG' => $this->environment->getNrOrg(),
          'CDFILIAL' => $cdfilial,
          'NRCARDAPIO' => $nrcardapio
      );

      try {
          $pstmt = $this->connection->prepare($sql);
          $pstmt->execute($params);
          return $pstmt->fetch(\PDO::FETCH_ASSOC)['CDFILANCESTQ'];
      } catch (\Exception $e) {
          throw $e;
      }
    }
    
    public function dadosLancamento($cdfilial, $nrcardapio, $nrretirada){
        $sql = "SELECT CDFILANCESTQ, NRLANCESTQ
                FROM RETIRADA
                WHERE RETIRADA.CDFILIAL = :CDFILIAL
                  AND RETIRADA.NRCARDAPIO = :NRCARDAPIO
                  AND RETIRADA.NRRETIRADA = :NRRETIRADA";
        $params = array(
            'CDFILIAL' => $cdfilial,
            'NRCARDAPIO' => $nrcardapio,
            'NRRETIRADA' => $nrretirada
        );

        try {
            $pstmt = $this->connection->prepare($sql);
            $pstmt->execute($params);
            $resultado = $pstmt->fetch();
            if($resultado){
                return [
                    'CDFILANCESTQ' => $resultado['CDFILANCESTQ'], 
                    'NRLANCESTQ' => $resultado['NRLANCESTQ']
                ];
            }
            return ['CDFILANCESTQ' => '', 'NRLANCESTQ' => ''];
      } catch (\Exception $e) {
          throw $e;
      }
    }
    
    private function isNewProduct($row){
        return isset($row['NEWPRODUCT']) && $row['NEWPRODUCT'];
    }
    
    public function isValidStock($row, &$error, $nrretirada = null, $nrcardapio = null, $nrsequitreti = null) {
        //if(isset($row['VALIDAR_ESTOQUE']) && $row['VALIDAR_ESTOQUE'] == false){
        //    return true;
        //}
        $CODIGO_MSG = '';
        $query = "SELECT P.CDPRODUTO, P.NMPRODUTO, P.CDARVPROD, P.SGUNIDADE, IDCNTRESTOQ, P.VRFATOCONV, TRUNC((D.QTESTOQDIA/P.VRFATOCONV), 5) AS QTESTOQDIA, P.CDPRODESTO, PF.IDPERMESTNEG, PF.IDPERMVISPOSEST, CUSTO_PRODUTO_LCEST(:P_CDFILIAL, :P_CDPRODUTO, TO_NUMBER(NVL(PF.CDTIPCUSTEST, '3')), TO_DATE(:P_DTPOSICAO), SYSDATE, :P_CDALMOXARIFE,:P_CDLOCALESTOQ, :P_NRLOTEESTQ, :P_NRSUBLOTE) VRCUSTPROD FROM PRODUTO P, POSESTDIA D, PRODFILI F, PARAMFILIAL PF, PARAMESTOQUE PE WHERE (P.CDPRODUTO = :P_CDPRODUTO) AND (D.CDFILIAL = :P_CDFILIAL) AND (D.CDALMOXARIFE = :P_CDALMOXARIFE) AND (D.CDLOCALESTOQ = :P_CDLOCALESTOQ) AND (D.NRLOTEESTQ = :P_NRLOTEESTQ) AND (D.NRSUBLOTE = :P_NRSUBLOTE) AND (D.CDPRODUTO = P.CDPRODESTO) AND (P.CDPRODESTO = F.CDPRODUTO) AND (F.CDFILIAL = :P_CDFILIAL) AND (F.CDFILIAL = PF.CDFILIAL) AND (DTPOSIESTQ = (SELECT MAX(DTPOSIESTQ) FROM POSESTDIA WHERE (CDFILIAL = :P_CDFILIAL) AND (CDALMOXARIFE = :P_CDALMOXARIFE) AND (CDLOCALESTOQ = :P_CDLOCALESTOQ) AND (NRLOTEESTQ = :P_NRLOTEESTQ) AND (NRSUBLOTE = :P_NRSUBLOTE) AND (CDPRODUTO = P.CDPRODESTO) AND (DTPOSIESTQ <= TO_DATE(:P_DTPOSICAO) + 1)))";
        $stmt = $this->connection->prepare($query);
        $params = array(
            "P_CDFILIAL"      => $row['CDFILIAL'],
            "P_CDPRODUTO"     => $row['CDPRODUTO'],
            "P_CDALMOXARIFE"  => $row['CDALMOXARIFE'],
            "P_CDLOCALESTOQ"  => $row['CDLOCALESTOQ'],
            "P_DTPOSICAO"     => $row['DTRETIRADA'],
            "P_NRLOTEESTQ"    => $row['NRLOTEESTQ'],
            "P_NRSUBLOTE"     => $row['NRSUBLOTE']
        );
        
        $stmt->execute($params);
        $estoque = $stmt->fetch(\PDO::FETCH_ASSOC);
        
        $evoluidoEstoque = 'N';
        $qtJaLancado = 0;
        if($nrsequitreti != null){
            $query = "SELECT IE.IDEVOLESTOQ, IE.QTTOTLANCTO FROM ITEMRETI IR INNER JOIN ITLANCTOEST IE ON IE.CDFILIAL = IR.CDFIITLANEST AND IE.NRLANCESTQ = IR.NRLANCESTQ AND IE.NRSEQUITEM = IR.NRSEQUITEM WHERE IR.CDFILIAL = :CDFILIAL AND IR.NRCARDAPIO = :NRCARDAPIO AND IR.NRRETIRADA = :NRRETIRADA AND IR.NRSEQUITRETI = :NRSEQUITRETI AND IE.CDALMOXARIFE = :CDALMOXARIFE AND IE.CDLOCALESTOQ = :CDLOCALESTOQ AND IE.NRLOTEESTQ = :NRLOTEESTQ AND IE.NRSUBLOTE = :NRSUBLOTE";
            $stmt = $this->connection->prepare($query);
            $params = array(
                "CDFILIAL" => $row['CDFILIAL'],
                "NRCARDAPIO" => $nrcardapio,
                "NRRETIRADA" => $nrretirada,
                "NRSEQUITRETI"  => $nrsequitreti,
                "CDALMOXARIFE"  => $row['CDALMOXARIFE'],
                "CDLOCALESTOQ"  => $row['CDLOCALESTOQ'],
                "NRLOTEESTQ"    => $row['NRLOTEESTQ'],
                "NRSUBLOTE"     => $row['NRSUBLOTE']
            );
            
            $stmt->execute($params);
            $prodReti = $stmt->fetch(\PDO::FETCH_ASSOC);
            if($prodReti){
                if($prodReti["IDEVOLESTOQ"] != null){
                    $evoluidoEstoque = $prodReti["IDEVOLESTOQ"];
                    $qtJaLancado = $prodReti["QTTOTLANCTO"];
                }
            }
        }
        
        if(!$estoque){
            $row['VRUNILANCTO'] = 0;
            $qtestoqdia = 0;
        }
        else {
            $row['VRUNILANCTO'] = $estoque['VRCUSTPROD'];
            $qtestoqdia = $estoque['QTESTOQDIA'];
        }
        if($evoluidoEstoque == 'S'){
            $qtestoqdia = $qtestoqdia + $qtJaLancado;
        }
        $qtestoqdia = round($qtestoqdia, 5);
        
        if($qtestoqdia >= $row['QTRETIRADA']){
            return true;
        }
        //  VALIDAÇÃO PARA EXIBIÇÃO DE SALDO EM ESTOQUE (QTESTOQDIA) ESTÁ UTILIZANDO APENAS O PARAMETRO IDEXIBESALDOESTRETI, O PARAMETRO IDPERMVISPOSEST FOI IGNORADO NO MOBILE;
        
        if($qtestoqdia <= 0){
            $CODIGO_MSG = 'PLA_MSG_PRODUTO_SEM_SALDO_EST';
        }else if($qtestoqdia < $row['QTRETIRADA']){
            if($row['IDEXIBESALDOESTRETI'] === 'S'){
                $CODIGO_MSG = 'PLA_MSG_QTD_NAO_DISPONIVEL_EST_EXIBE_SALDO#' .$qtestoqdia;
            } else {
                $CODIGO_MSG = 'PLA_MSG_QTD_NAO_DISPONIVEL_EST';
            }
        }
        if($error !== null && $CODIGO_MSG) {
            array_push($error, $CODIGO_MSG);
        }
        if(!$CODIGO_MSG) return true;
        return false;
    }
    
    public function getCustoTotal($cdfilial, $nrretirada, $nrcardapio){
        $query = "SELECT SUM(NVL(IT.VRTOTLANCTO, 0.0)) AS CUSTOTOTAL FROM RETIRADA R INNER JOIN ITLANCTOEST IT ON IT.CDFILIAL = R.CDFILANCESTQ AND IT.NRLANCESTQ = R.NRLANCESTQ WHERE R.CDFILIAL = :CDFILIAL AND R.NRRETIRADA = :NRRETIRADA AND R.NRCARDAPIO = :NRCARDAPIO";
        $stmt = $this->connection->prepare($query);
        $params = array(
            "CDFILIAL"      => $cdfilial,
            "NRRETIRADA"     => $nrretirada,
            "NRCARDAPIO"  => $nrcardapio
        );
        
        $stmt->execute($params);
        return $stmt->fetch(\PDO::FETCH_ASSOC);
    }
    
    public function getProdutosNaoEvoluidos($cdfilial, $nrretirada, $nrcardapio){
        $query = "SELECT DISTINCT FORMATPRODMASK(PT.CDARVPROD) || ' ' || PT.NMPRODUTO AS PRODUTO
                  FROM ITEMRETI I
                       INNER JOIN ITLANCTOEST E ON E.CDFILIAL = I.CDFIITLANEST AND
                                                   E.NRLANCESTQ = I.NRLANCESTQ AND
                                                   E.NRSEQUITEM = I.NRSEQUITEM
                       INNER JOIN PRODUTO PT ON PT.CDPRODUTO = E.CDPRODUTO
                 WHERE I.CDFILIAL = :CDFILIAL
                   AND I.NRRETIRADA = :NRRETIRADA
                   AND I.NRCARDAPIO = :NRCARDAPIO
                   AND E.IDEVOLESTOQ = 'N'";
        $stmt = $this->connection->prepare($query);
        $params = array(
            "CDFILIAL"      => $cdfilial,
            "NRRETIRADA"     => $nrretirada,
            "NRCARDAPIO"  => $nrcardapio
        );
        $stmt->execute($params);
        return $stmt->fetchAll();
    }
}